from django.core.management.base import BaseCommand, CommandError

from bulbs.promotion.models import PZone


pzone_names = [pzone.name for pzone in PZone.objects.all()]


class Command(BaseCommand):

    help = "Set the length of a pzone."

    def add_arguments(self, parser):

        parser.add_argument(
            "zone_name",
            help="Name of zone to set the zone_length for.",
            choices=pzone_names,
            type=str)

        parser.add_argument(
            "zone_length",
            help="Pzone's new zone_length.",
            type=int)

    def handle(self, *args, **options):

        zone_length = options["zone_length"]
        if zone_length < 0:
            raise CommandError("zone_length must be a positive number.")

        pzone = PZone.objects.get(name=options["zone_name"])
        pzone.zone_length = zone_length
        pzone.save()

        print("PZone '{}' length updated to {}".format(pzone.name, pzone.zone_length))



from django.core.exceptions import ObjectDoesNotExist

from celery import shared_task


@shared_task(default_retry_delay=5)
def index(content_type_id, pk, refresh=False):
    from django.contrib.contenttypes.models import ContentType
    content_type = ContentType.objects.get_for_id(content_type_id)
    obj = content_type.model_class().objects.get(id=pk)
    obj.index(refresh=refresh)


@shared_task(default_retry_delay=5)
def index_content_contributions(content_pk):
    from bulbs.contributions.models import Contribution
    for contribution in Contribution.objects.filter(content__pk=content_pk):
        contribution.save()


@shared_task(default_retry_delay=5)
def index_content_report_content_proxy(content_pk):
    from bulbs.contributions.models import ReportContent
    try:
        proxy = ReportContent.reference.get(id=content_pk)
        proxy.index()
    except ObjectDoesNotExist:
        pass



from collections import OrderedDict
import datetime
import six

from django.contrib.auth import get_user_model
from django.shortcuts import get_object_or_404
from django.template.defaultfilters import slugify
from django.utils import dateparse, timezone

from rest_framework import serializers
from rest_framework.utils import model_meta

from bulbs.content.models import Content, FeatureType
from bulbs.content.serializers import FeatureTypeField, UserSerializer

from .models import (
    Contribution, ContributorRole, ContributionOverride, HourlyRate, FlatRate, ManualRate,
    FeatureTypeRate, FeatureTypeOverride, LineItem, OverrideProfile, Rate,
    RATE_PAYMENT_TYPES, MANUAL
)


class PaymentTypeField(serializers.Field):
    """
    payment type objects serialized to/from label/identifer
    """
    def to_representation(self, obj):
        data = dict(RATE_PAYMENT_TYPES)[obj]
        return data

    def to_internal_value(self, data):
        if isinstance(data, int):
            return data
        return dict((label, value) for value, label in RATE_PAYMENT_TYPES)[data]


class ContributorSerializer(serializers.ModelSerializer):

    full_name = serializers.SerializerMethodField()

    class Meta:
        model = get_user_model()
        fields = ("id", "first_name", "last_name", "full_name")

    def get_full_name(self, contributor):
        if hasattr(contributor, "get_full_name"):
            return contributor.get_full_name()
        elif hasattr(contributor, "full_name"):
            return contributor.full_name
        else:
            return contributor.first_name + " " + contributor.last_name


class ContributorField(serializers.Field):

    def to_representation(self, obj):
        return ContributorSerializer(obj).data

    def to_internal_value(self, data):
        contributor_cls = get_user_model()

        if isinstance(data, six.text_type) and data.isdigit():
            data = int(data)

        if isinstance(data, int):
            return contributor_cls.objects.get(id=data)
        elif isinstance(data, dict):
            id = data.get("id", None)
            if id is not None:
                return contributor_cls.objects.get(id=id)
        return None


class FreelanceProfileSerializer(serializers.Serializer):

    contributor = ContributorField()
    payment_date = serializers.SerializerMethodField()
    pay = serializers.SerializerMethodField()
    contributions_count = serializers.SerializerMethodField("get_contribution_count")

    def get_pay(self, obj):
        return obj.get_pay()

    def get_payment_date(self, obj):
        now = timezone.now()
        month = now.month + 1
        year = now.year
        if month > 12:
            month = 1
            year += 1
        next_payment = datetime.datetime(
            day=1,
            month=month,
            year=year,
            tzinfo=now.tzinfo
        )
        return next_payment.isoformat()

    def get_contribution_count(self, obj):
        return obj.contributor.contributions.count()


class LineItemSerializer(serializers.ModelSerializer):

    contributor = ContributorField()
    payment_date = serializers.DateTimeField()

    class Meta:
        model = LineItem


class RateSerializer(serializers.ModelSerializer):

    name = PaymentTypeField()

    class Meta:
        model = Rate


class NestedRateSerializer(serializers.ModelSerializer):
    """
    ModelSerializer that lookups the user role from the url kwargs.
    """
    def create(self, validated_data):
        lookup_kwargs = self.context["request"].parser_context["kwargs"]
        role_pk = lookup_kwargs.get("role_pk", 0)
        validated_data["role"] = get_object_or_404(ContributorRole, pk=role_pk)
        return super(NestedRateSerializer, self).create(validated_data)


class FlatRateSerializer(NestedRateSerializer):

    class Meta:
        model = FlatRate
        fields = ("id", "rate")


class HourlyRateSerializer(NestedRateSerializer):

    class Meta:
        model = HourlyRate
        fields = ("id", "rate")


class FeatureTypeRateSerializer(NestedRateSerializer):

    feature_type = FeatureTypeField(queryset=FeatureType.objects.all())

    class Meta:
        model = FeatureTypeRate
        fields = ("id", "rate", "feature_type")


class RateField(serializers.Field):
    """
    Returns the appropriate rate to represent
    Creates a new rate
    """
    def get_attribute(self, obj):
        return obj.get_rate()

    def to_representation(self, obj):
        return RateSerializer(obj).data

    def to_internal_value(self, data):
        name = data.get('name', None)
        if name:
            data['name'] = dict((label, value) for value, label in RATE_PAYMENT_TYPES)[name]
        if 'role' in data:
            rate = FlatRate(**data)
        elif 'contribution' in data:
            data["name"] = MANUAL
            rate = ManualRate(**data)
        elif 'feature_type' in data:
            rate = FeatureTypeRate(**data)
        else:
            rate = Rate(**data)
        rate.save()
        return rate


class OverrideRateField(serializers.Field):

    def get_attribute(self, obj):
        return obj.override_flatrate.first()

    def to_representation(self, obj):
        return obj.rate


class OverrideFeatureTypesField(serializers.Field):

    def get_attribute(self, obj):
        return obj.override_feature_type.all()

    def to_representation(self, obj):
        marshaller = []
        for rate in obj:
            marshaller.append({
                'feature_type': rate.feature_type.name,
                'rate': rate.rate
            })
        return marshaller


class ContributorRoleSerializer(serializers.ModelSerializer):

    payment_type = PaymentTypeField()
    rate = serializers.SerializerMethodField()

    class Meta:
        model = ContributorRole

    def get_rate(self, obj):
        rate = obj.get_rate()
        if rate:
            return rate.rate
        return None


class RoleField(serializers.Field):

    def get_attribute(self, obj):
        return (obj.role, obj)

    def to_representation(self, obj):
        obj, contribution = obj
        data = {
            "id": obj.id,
            "name": obj.name,
            "description": obj.description,
            "payment_type": PaymentTypeField().to_representation(obj.payment_type)
        }

        if isinstance(contribution, Contribution):
            rate = data.get("rate")
            if not rate:
                rate = contribution.get_rate()
                if rate and hasattr(rate, "rate"):
                    data["rate"] = rate.rate

            # TODO: Need to separate these rate requests to a different endpoint
            content = getattr(contribution, 'content', None)
            if content is not None:

                data["rates"] = {}

                # TODO: This is redundant, but we need to lower the feature_type query on partial.
                flat_rate = FlatRate.objects.filter(role=obj).order_by("-updated_on").first()
                if flat_rate:
                    data["rates"]["flat_rate"] = {
                        "rate": flat_rate.rate,
                        "updated_on": flat_rate.updated_on.isoformat()
                    }

                hourly = HourlyRate.objects.filter(role=obj).order_by("-updated_on").first()
                if hourly:
                    data["rates"]["hourly"] = {
                        "rate": hourly.rate,
                        "updated_on": hourly.updated_on.isoformat()
                    }

                feature_type_rates = FeatureTypeRate.objects.filter(
                    role=obj, feature_type=content.feature_type
                )
                if feature_type_rates.exists():
                    ft = feature_type_rates.order_by("-updated_on")[0]
                    data["rates"]["feature_type"] = [{
                        "feature_type": ft.feature_type.name,
                        "rate": ft.rate,
                        "updated_on": ft.updated_on.isoformat()
                    }]
        return data

    def to_internal_value(self, data):
        if isinstance(data, int):
            return ContributorRole.objects.get(id=data)
        elif isinstance(data, dict):
            id = data.get("id", None)
            return ContributorRole.objects.get(id=id)
        return None


class FeatureTypeOverrideSerializer(serializers.ModelSerializer):

    rate = serializers.IntegerField(required=False)
    feature_type = FeatureTypeField(queryset=FeatureType.objects.all())

    class Meta:
        model = FeatureTypeOverride


class OverrideProfileSerializer(serializers.ModelSerializer):

    contributor = ContributorField()
    role = RoleField()
    rate = OverrideRateField(read_only=True)
    feature_types = OverrideFeatureTypesField(read_only=True)

    class Meta:
        model = OverrideProfile

    def update_feature_type_overrides(self, profile, feature_types):
        for data in feature_types:
            ft_data = data.get('feature_type', None)
            if ft_data:
                feature_type = FeatureTypeField(read_only=True).to_internal_value(
                    ft_data
                )
                rate = int(data.get('rate', 0))

                ft_override = profile.override_feature_type.get_or_create(
                    feature_type=feature_type,
                )[0]
                if ft_override.rate != rate:
                    ft_override.rate = rate
                    ft_override.save()

    def to_internal_value(self, data):
        rate = data.pop("rate", None)
        feature_types = data.pop("feature_types", None)
        override = super(OverrideProfileSerializer, self).to_internal_value(data)
        if feature_types:
            override['feature_types'] = feature_types
        if rate:
            override['rate'] = rate
        return override

    def create(self, validated_data):
        rate = validated_data.pop('rate', None)
        feature_types = validated_data.pop('feature_types', [])
        profile = super(OverrideProfileSerializer, self).create(validated_data)

        if rate or rate == 0:
            profile.override_flatrate.create(rate=rate)

        if feature_types:
            self.update_feature_type_overrides(profile, feature_types)

        return profile

    def update(self, instance, validated_data):
        rate = validated_data.pop('rate', None)
        feature_types = validated_data.pop('feature_types', [])
        profile = super(OverrideProfileSerializer, self).update(instance, validated_data)

        if rate or rate == 0:
            profile.override_flatrate.create(rate=rate)

        if feature_types:
            self.update_feature_type_overrides(profile, feature_types)

        return profile


class ContributionListSerializer(serializers.ListSerializer):

    contributor = UserSerializer()

    def update(self, instance, validated_data):
        # Maps for id->instance and id->data item.
        contribution_mapping = {c.id: c for c in instance}
        data_mapping = {item['id']: item for item in validated_data if "id" in item}

        # Perform creations and updates.
        ret = []
        for data in validated_data:
            contribution = None
            if "id" in data:
                contribution = contribution_mapping.get(data["id"], None)

            if contribution is None:
                ret.append(self.child.create(data))
            else:
                ret.append(self.child.update(contribution, data))

        # Perform deletions.
        for contribution_id, contribution in contribution_mapping.items():
            if contribution_id not in data_mapping:
                contribution.delete()

        return ret


class ContributionOverrideField(serializers.Field):

    def get_attribute(self, obj):
        return obj.override_contribution.first()

    def to_representation(self, obj):
        if obj:
            return obj.rate


class ContributionSerializer(serializers.ModelSerializer):

    contributor = UserSerializer()
    rate = RateField(required=False)
    role = RoleField()
    override_rate = ContributionOverrideField(read_only=True)
    content = serializers.PrimaryKeyRelatedField(queryset=Content.objects.all())

    class Meta:
        model = Contribution
        list_serializer_class = ContributionListSerializer

    def get_rate(self, obj):
        rate = obj.get_rate()
        if not rate:
            return None
        return RateSerializer(rate).data

    def to_internal_value(self, data):
        rate_data = data.pop('rate', None)
        override_rate_data = data.pop("override_rate", None)
        data = super(ContributionSerializer, self).to_internal_value(data)
        if rate_data:
            data['rate'] = rate_data
        if override_rate_data:
            data["override_rate"] = override_rate_data
        return data

    def create(self, validated_data):
        model_cls = self.Meta.model
        info = model_meta.get_field_info(model_cls)
        many_to_many = {}
        rate_data = validated_data.pop('rate', None)
        override_rate_data = validated_data.pop("override_rate", None)
        for field_name, relation_info in info.relations.items():
            if relation_info.to_many and (field_name in validated_data):
                many_to_many[field_name] = validated_data.pop(field_name)
        contribution = model_cls.objects.create(**validated_data)
        if rate_data:
            if isinstance(rate_data, int):
                rate_data = {"rate": rate_data}
            if isinstance(rate_data, six.text_type) or isinstance(rate_data, str):
                if rate_data.isdigit():
                    rate_data = {"rate": int(rate_data)}
            rate_data["contribution"] = contribution
            RateField().to_internal_value(rate_data)
        if override_rate_data:
            ContributionOverride.objects.create(
                contribution=contribution,
                rate=override_rate_data
            )
        return contribution


class ContributionReportingSerializer(serializers.ModelSerializer):

    user = serializers.SerializerMethodField()
    content = serializers.SerializerMethodField()
    pay = serializers.SerializerMethodField()
    role = serializers.SerializerMethodField()
    rate = serializers.SerializerMethodField()

    class Meta:
        model = Contribution
        fields = ("id", "content", "user", "pay", "role", "rate", "notes")

    def get_content(self, obj):
        return OrderedDict([
            ("id", obj.content.id),
            ("title", obj.content.title),
            ("url", obj.content.get_absolute_url()),
            ("content_type", obj.content.__class__.__name__),
            ("feature_type", getattr(obj.content.feature_type, "name", None)),
            ("published", timezone.localtime(obj.content.published))
        ])

    def get_user(self, obj):
        full_name = obj.contributor.get_full_name()
        data = {
            "id": obj.contributor.id,
            "username": obj.contributor.username,
            "full_name": full_name,
            "payroll_name": full_name
        }
        profile = getattr(obj.contributor, "freelanceprofile", None)
        if profile:
            payroll_name = getattr(profile, "payroll_name")
            if payroll_name:
                data["payroll_name"] = payroll_name
        return data

    def get_role(self, obj):
        return obj.role.name

    def get_pay(self, obj):
        return obj.get_pay

    def get_rate(self, obj):
        rate = obj.get_rate()
        if isinstance(rate, HourlyRate):
            minutes_worked = getattr(obj, 'minutes_worked', 0)
            return rate.rate * minutes_worked
        if rate and hasattr(rate, "rate"):
            return rate.rate
        return None


class ContributorRoleField(serializers.Field):
    """This is fucking stupid, but it's basically a field that returns the the
    names of people who have contributed to this content under a certain role."""

    def __init__(self, role, *args, **kwargs):
        super(ContributorRoleField, self).__init__(*args, **kwargs)
        self.role = role
        self.source = "*"

    def to_representation(self, obj):
        qs = Contribution.objects.filter(content=obj, role=self.role).select_related("contributor")
        return ",".join([contribution.contributor.get_full_name() for contribution in qs])


class ContentReportingSerializer(serializers.ModelSerializer):

    content_type = serializers.SerializerMethodField()
    published = serializers.SerializerMethodField()
    feature_type = serializers.SerializerMethodField()
    url = serializers.URLField(source="get_absolute_url")
    value = serializers.SerializerMethodField('get_content_value')
    authors = serializers.SerializerMethodField()
    video_id = serializers.SerializerMethodField()

    class Meta:
        model = Content
        fields = (
            "id", "title", "url", "content_type", "feature_type", "published", "authors",
            "video_id", "value"
        )

    def get_content_value(self, obj):
        try:
            contributions = obj.contributions.distinct()
        except:
            contributions = obj.contributions.instances
        request = self.context.get("request")
        now = timezone.now()
        start_date = datetime.datetime(
            year=now.year,
            month=now.month,
            day=1,
            tzinfo=now.tzinfo
        )
        if "start" in request.QUERY_PARAMS:
            start_date = dateparse.parse_date(request.QUERY_PARAMS["start"])

        end_date = now
        if "end" in request.GET:
            end_date = dateparse.parse_date(request.QUERY_PARAMS["end"])

        if "contributors" in request.QUERY_PARAMS:
            contributors = request.QUERY_PARAMS.getlist("contributors")
            contributions = contributions.filter(contributor__username__in=contributors)

        # contributions = contributions.filter(
        #     force_payment=False
        # ) | contributions.filter(
        #     payment_date__range=(start_date, end_date)
        # )

        total_cost = 0
        for contribution in contributions:
            cost = contribution.get_pay
            if cost:
                total_cost += cost
        return total_cost

    def get_content_type(self, obj):
        return obj.__class__.__name__

    def get_feature_type(self, obj):
        return getattr(obj.feature_type, "name", None)

    def get_video_id(self, obj):
        if hasattr(obj, 'videohub_ref'):
            return getattr(obj.videohub_ref, 'id')
        elif hasattr(obj, 'video'):
            return getattr(obj.video, 'id')
        return None

    def get_published(self, obj):
        return timezone.localtime(obj.published)

    def get_authors(self, obj):
        return ",".join([author.get_full_name() for author in obj.authors.all()])



"""ElasticSearch filters specifially for content"""

import datetime
import dateutil.parser
import dateutil.tz
from django.utils import timezone

from elasticsearch_dsl.filter import Exists, MatchAll, Nested, Not, Range, Term, Terms
from elasticsearch_dsl.query import FunctionScore

from six import string_types, text_type, binary_type


def Evergreen(evergreen=True):
    return Term(evergreen=evergreen)


def parse_datetime(value):
    """Returns a datetime object for a given argument

    This helps to convert strings, dates and datetimes to proper tz-enabled
    datetime objects."""

    if isinstance(value, (string_types, text_type, binary_type)):
        value = dateutil.parser.parse(value)
        value.replace(tzinfo=dateutil.tz.tzutc())
        return value
    elif isinstance(value, datetime.datetime):
        value.replace(tzinfo=dateutil.tz.tzutc())
        return value
    elif isinstance(value, datetime.date):
        value = datetime.datetime(value.year, value.month, value.day)
        value.replace(tzinfo=dateutil.tz.tzutc())
        return value
    else:
        raise ValueError('Value must be parsable to datetime object. Got `{}`'.format(type(value)))


def _parse_slugs(slugs):
    included = []
    excluded = []
    for slug in slugs:
        if slug.startswith("-"):
            excluded.append(slug[1:])
        else:
            included.append(slug)

    return (included, excluded)


def Published(before=None, after=None):  # noqa
    published_params = {}
    if after is not None:
        published_params["gte"] = parse_datetime(after)

    if before is not None:
        published_params["lte"] = parse_datetime(before)

    if before is None and after is None:
        published_params["lte"] = timezone.now()

    return Range(published=published_params)


def Status(status):  # noqa
    if status:
        return Term(status=status)
    else:
        return MatchAll()


def AllSponsored():  # noqa
    """Filter for any content that has a campaign."""
    return Nested(path="campaign", filter=Exists(field="campaign.id"))


def Authors(usernames):  # noqa
    included, excluded = _parse_slugs(usernames)

    f = MatchAll()
    if included:
        f &= Terms(**{"authors.username": included})
    if excluded:
        f &= Terms(**{"authors.username": excluded})
    return f


def NegateQueryFilter(es_query):  # noqa
    """
    Return a filter removing the contents of the provided query.
    """
    query = es_query.to_dict().get("query", {})
    filtered = query.get("filtered", {})
    negated_filter = filtered.get("filter", {})
    return Not(**negated_filter)


def Tags(slugs):  # noqa
    included, excluded = _parse_slugs(slugs)

    f = MatchAll()
    if included:
        f &= Nested(path="tags", filter=Terms(**{"tags.slug": included}))
    if excluded:
        f &= ~Nested(path="tags", filter=Terms(**{"tags.slug": excluded}))

    return f


def FeatureTypes(slugs):  # noqa
    included, excluded = _parse_slugs(slugs)

    f = MatchAll()
    if included:
        f &= Nested(path="feature_type", filter=Terms(**{"feature_type.slug": included}))
    if excluded:
        f &= ~Nested(path="feature_type", filter=Terms(**{"feature_type.slug": excluded}))

    return f


def SponsoredBoost(field_name, boost_mode="multiply", weight=5):
    return FunctionScore(
        boost_mode=boost_mode,
        functions=[{
            "filter": Exists(field=field_name),
            "weight": weight
        }]
    )


def VideohubChannel(included_ids=None, excluded_ids=None):
    f = MatchAll()
    if included_ids:
        f &= Nested(path="video", filter=Terms(**{"video.channel_id": included_ids}))
    if excluded_ids:
        f &= ~Nested(path="video", filter=Terms(**{"video.channel_id": excluded_ids}))


def TagBoost(slugs, boost_mode="multiply", weight=5):
    included, excluded = _parse_slugs(slugs)
    return FunctionScore(
        boost_mode=boost_mode,
        functions=[{
            "filter": Nested(path="tags", filter=Terms(**{"tags.slug": included})),
            "weight": weight
        }]
    )


def FeatureTypeBoost(slugs, boost_mode="multiply", weight=5):
    included, excluded = _parse_slugs(slugs)
    return FunctionScore(
        boost_mode=boost_mode,
        functions=[{
            "filter": Nested(path="feature_type", filter=Terms(**{"feature_type.slug": included})),
            "weight": weight
        }]
    )



from django.utils import timezone

from bulbs.utils.filters import CaseInsensitiveBooleanFilter
from bulbs.utils.methods import get_query_params


class SpecialCoverageFilter(CaseInsensitiveBooleanFilter):
    """
    Includes the default boolean_filter, but replaces the active and inactive states to represent
    whether the current_date is within a SpecialCoverage's start_date & end_date range.
    """
    def filter_queryset(self, request, queryset, view):
        queryset = super(SpecialCoverageFilter, self).filter_queryset(request, queryset, view)
        query_params = get_query_params(request)
        if "active" in query_params:
            today_filter = timezone.now()
            value = query_params.get("active").lower()
            if value == "true":
                queryset = queryset.filter(
                    start_date__lte=today_filter, end_date__gte=today_filter
                )
            elif value == "false":
                queryset == queryset.exclude(
                    start_date__lte=today_filter, end_date__gte=today_filter
                )
        return queryset



from celery import shared_task

from .models import Contribution


@shared_task(default_retry_delay=5)
def update_role_rates(contributor_role_pk):
    for contribution in Contribution.objects.filter(contributor__pk=contributor_role_pk):
        contribution.index()



"""API Views and ViewSets"""

import datetime

from django.utils import dateparse, timezone

from elasticsearch_dsl import filter as es_filter
from rest_framework import viewsets, routers, mixins
from rest_framework.settings import api_settings
from rest_framework.response import Response
from rest_framework_csv.renderers import CSVRenderer
from rest_framework_nested import routers as nested_routers

from bulbs.content.filters import FeatureTypes, Published, Tags


from .filters import ESPublishedFilterBackend, StartEndFilterBackend
from .models import (
    ContributorRole, Contribution, FeatureTypeRate, FlatRate, FreelanceProfile, HourlyRate,
    LineItem, OverrideProfile, ReportContent, MANUAL
)
from .renderers import ContributionReportingRenderer
from .csv_serializers import ContributionCSVSerializer
from .serializers import (
    ContributorRoleSerializer, ContributionReportingSerializer, ContentReportingSerializer,
    FeatureTypeRateSerializer, FlatRateSerializer, FreelanceProfileSerializer,
    HourlyRateSerializer, LineItemSerializer, OverrideProfileSerializer
)
from .utils import get_forced_payment_contributions


class LineItemViewSet(viewsets.ModelViewSet):
    queryset = LineItem.objects.all()
    serializer_class = LineItemSerializer
    start_fields = ("payment_date",)
    end_fields = ("payment_date",)
    filter_backends = (StartEndFilterBackend,)
    paginate_by = 20


class ContributorRoleViewSet(viewsets.ModelViewSet):
    serializer_class = ContributorRoleSerializer

    def get_queryset(self):
        queryset = ContributorRole.objects.all()
        if self.request.QUERY_PARAMS.get("override", None) == "true":
            queryset = queryset.exclude(payment_type=MANUAL)
        return queryset


class NestedRateViewSet(viewsets.ModelViewSet):
    """filters queryset by role given the role_pk url kwarg"""

    def get_queryset(self):
        role_pk = self.request.parser_context["kwargs"].get("role_pk")
        return self.model.objects.filter(role__pk=role_pk)


class FlatRateViewSet(NestedRateViewSet):

    model = FlatRate
    serializer_class = FlatRateSerializer
    paginate_by = 20


class HourlyRateViewSet(NestedRateViewSet):

    model = HourlyRate
    serializer_class = HourlyRateSerializer
    paginate_by = 20


class FeatureTypeRateViewSet(NestedRateViewSet):

    model = FeatureTypeRate
    serializer_class = FeatureTypeRateSerializer
    paginate_by = 20


class OverrideProfileViewSet(viewsets.ModelViewSet):

    model = OverrideProfile
    queryset = OverrideProfile.search_objects.all()
    serializer_class = OverrideProfileSerializer


class ContentReportingViewSet(viewsets.GenericViewSet, mixins.ListModelMixin):

    renderer_classes = (CSVRenderer, ) + tuple(api_settings.DEFAULT_RENDERER_CLASSES)
    serializer_class = ContentReportingSerializer
    filter_backends = (ESPublishedFilterBackend,)
    paginate_by = 20

    def get_queryset(self):
        qs = ReportContent.search_objects.search()

        # TODO: reintroduce forced submissions
        # include, exclude = get_forced_payment_contributions(start_date, end_date)
        # include_ids = include.values_list("content__id", flat=True)
        # exclude_ids = exclude.values_list("content__id", flat=True)

        # content = Content.objects.filter(
        #     contributions__gt=0
        # ).filter(
        #     published__range=(start_date, end_date)
        # ).exclude(
        #     pk__in=exclude_ids
        # ).prefetch_related(
        #     "authors", "contributions"
        # ).select_related(
        #     "feature_type"
        # ).distinct() | Content.objects.filter(pk__in=include_ids).distinct()

        if "feature_types" in self.request.QUERY_PARAMS:
            feature_types = self.request.QUERY_PARAMS.getlist("feature_types")
            qs = qs.filter(FeatureTypes(feature_types))

        if "tags" in self.request.QUERY_PARAMS:
            tags = self.request.QUERY_PARAMS.getlist("tags")
            qs = qs.filter(Tags(tags))

        # if "staff" in self.request.QUERY_PARAMS:
        #     staff = self.request.QUERY_PARAMS.get("staff")
        #     if staff == "freelance":
        #         contribution_content_ids = Contribution.objects.filter(
        #             contributor__freelanceprofile__is_freelance=True
        #         ).values_list(
        #             "content__id", flat=True
        #         ).distinct()
        #     elif staff == "staff":
        #         contribution_content_ids = Contribution.objects.filter(
        #             contributor__freelanceprofile__is_freelance=False
        #         ).values_list(
        #             "content__id", flat=True
        #         ).distinct()
        #     if contribution_content_ids:
        #         content = content.filter(pk__in=contribution_content_ids)

        if "contributors" in self.request.QUERY_PARAMS:
            contributors = self.request.QUERY_PARAMS.getlist("contributors")
            qs = qs.filter(
                es_filter.Terms(**{'contributions.contributor.username': contributors})
            )
        #     contribution_content_ids = Contribution.objects.filter(
        #         contributor__username__in=contributors
        #     ).values_list(
        #         "content__id", flat=True
        #     ).distinct()
        #     content = content.filter(pk__in=contribution_content_ids)

        return qs


class ReportingViewSet(viewsets.GenericViewSet, mixins.ListModelMixin):

    renderer_classes = (
        ContributionReportingRenderer,
    ) + tuple(
        api_settings.DEFAULT_RENDERER_CLASSES
    )
    filter_backends = (ESPublishedFilterBackend,)
    paginate_by = 20

    def list(self, request):
        format = self.request.QUERY_PARAMS.get('format', None)
        if format == 'csv':
            queryset = self.get_queryset()
            # TODO: get rid of this csv pattern so it incorporates the typical restful patterns.
            queryset = ESPublishedFilterBackend().filter_queryset(request, queryset, None)
            queryset = queryset[:queryset.count()]
            serializer = self.get_serializer(queryset, many=True)
            return Response(serializer.data)
        return super(ReportingViewSet, self).list(request)

    def get_serializer_class(self):
        format = self.request.QUERY_PARAMS.get('format', None)
        if format == 'csv':
            return ContributionCSVSerializer
        return ContributionReportingSerializer

    def get_queryset(self):
        qs = Contribution.search_objects.search()

        feature_types = self.request.QUERY_PARAMS.getlist('feature_types')
        if feature_types:
            qs = qs.filter(
                es_filter.Terms(**{'content.feature_type.slug': feature_types})
            )

        contributors = self.request.QUERY_PARAMS.getlist('contributors')
        if contributors:
            qs = qs.filter(
                es_filter.Terms(**{'contributor.username': contributors})
            )

        tags = self.request.QUERY_PARAMS.getlist('tags')
        if tags:
            pass

        staff = self.request.QUERY_PARAMS.get('staff', None)
        if staff:
            is_freelance = True if staff == 'freelance' else False
            qs = qs.filter(
                es_filter.Term(**{'contributor.is_freelance': is_freelance})
            )
        return qs.sort('id')


class FreelanceReportingViewSet(viewsets.GenericViewSet, mixins.ListModelMixin):

    renderer_classes = (CSVRenderer, ) + tuple(api_settings.DEFAULT_RENDERER_CLASSES)
    serializer_class = FreelanceProfileSerializer
    paginate_by = 20

    def get_queryset(self):
        now = timezone.now()
        start_date = datetime.datetime(
            year=now.year,
            month=now.month,
            day=1,
            tzinfo=now.tzinfo
        )

        if "start" in self.request.GET:
            start_date = dateparse.parse_date(self.request.GET["start"])

        end_date = now
        if "end" in self.request.GET:
            end_date = dateparse.parse_date(self.request.GET["end"])

        contribution_qs = Contribution.objects.all()
        if "feature_types" in self.request.QUERY_PARAMS:
            feature_types = self.request.QUERY_PARAMS.getlist("feature_types")
            contribution_qs = contribution_qs.filter(content__feature_type__slug__in=feature_types)

        if "tags" in self.request.QUERY_PARAMS:
            tags = self.request.QUERY_PARAMS.getlist("tags")
            contribution_qs = contribution_qs.filter(content__tags__slug__in=tags)

        include, exclude = get_forced_payment_contributions(
            start_date, end_date, qs=contribution_qs
        )
        include_ids = include.values_list('pk', flat=True).distinct()
        exclude_ids = exclude.values_list('pk', flat=True).distinct()

        contribution_qs = contribution_qs.filter(
            content__published__range=(start_date, end_date)
        ) | Contribution.objects.filter(
            pk__in=include_ids
        )
        contribution_qs = contribution_qs.exclude(pk__in=exclude_ids).distinct()

        if "contributors" in self.request.QUERY_PARAMS:
            contributors = self.request.QUERY_PARAMS.getlist("contributors")
            contribution_qs = contribution_qs.filter(contributor__username__in=contributors)

        contributor_ids = contribution_qs.values_list('contributor', flat=True).distinct()
        qs = FreelanceProfile.objects.filter(contributor__in=contributor_ids)

        if "staff" in self.request.QUERY_PARAMS:
            staff = self.request.QUERY_PARAMS.get("staff")
            if staff == "freelance":
                qs = qs.filter(is_freelance=True)
            elif staff == "staff":
                qs = qs.filter(is_freelance=False)
        return qs


api_v1_router = routers.DefaultRouter()
api_v1_router.register(r"line-items", LineItemViewSet, base_name="line-items")

# Nested routes for roles
api_v1_role_router = nested_routers.DefaultRouter()
api_v1_role_router.register(r"role", ContributorRoleViewSet, base_name="contributorrole")
nested_role_router = nested_routers.NestedSimpleRouter(api_v1_role_router, "role", lookup="role")
nested_role_router.register("flat_rates", FlatRateViewSet, base_name="flat-rate")
nested_role_router.register("hourly_rates", HourlyRateViewSet, base_name="hourly-rate")
nested_role_router.register(
    "feature_type_rates", FeatureTypeRateViewSet, base_name="feature-type-rate"
)

api_v1_router.register(r"rate-overrides", OverrideProfileViewSet, base_name="rate-overrides")
api_v1_router.register(r"reporting", ReportingViewSet, base_name="contributionreporting")
api_v1_router.register(r"contentreporting", ContentReportingViewSet, base_name="contentreporting")
api_v1_router.register(
    r"freelancereporting", FreelanceReportingViewSet, base_name="freelancereporting"
)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


# Safe User import for Django < 1.5
try:
    from django.contrib.auth import get_user_model
except ImportError:
    from django.contrib.auth.models import User
else:
    User = get_user_model()
# from django.conf import settings
# from django.db.models.loading import get_model
# User = get_model(*settings.AUTH_USER_MODEL.split("."))


# With the default User model these will be 'auth.User' and 'auth.user'
# so instead of using orm['auth.User'] we can use orm[user_orm_label]
user_orm_label = '%s.%s' % (User._meta.app_label, User._meta.object_name)
user_model_label = '%s.%s' % (User._meta.app_label, User._meta.module_name)


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'ContentListOperation'
        db.create_table(u'promotion_contentlistoperation', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('polymorphic_ctype', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'polymorphic_promotion.contentlistoperation_set', null=True, to=orm['contenttypes.ContentType'])),
            ('content_list', self.gf('django.db.models.fields.related.ForeignKey')(related_name='operations', to=orm['promotion.ContentList'])),
            ('when', self.gf('django.db.models.fields.DateTimeField')()),
            ('applied', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'promotion', ['ContentListOperation'])

        # Adding model 'InsertOperation'
        db.create_table(u'promotion_insertoperation', (
            (u'contentlistoperation_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['promotion.ContentListOperation'], unique=True, primary_key=True)),
            ('index', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('content', self.gf('django.db.models.fields.related.ForeignKey')(related_name='+', to=orm['content.Content'])),
            ('lock', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'promotion', ['InsertOperation'])

        # Adding model 'ReplaceOperation'
        db.create_table(u'promotion_replaceoperation', (
            (u'contentlistoperation_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['promotion.ContentListOperation'], unique=True, primary_key=True)),
            ('content', self.gf('django.db.models.fields.related.ForeignKey')(related_name='+', to=orm['content.Content'])),
            ('target', self.gf('django.db.models.fields.related.ForeignKey')(related_name='+', to=orm['content.Content'])),
            ('lock', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'promotion', ['ReplaceOperation'])

        # Adding model 'LockOperation'
        db.create_table(u'promotion_lockoperation', (
            (u'contentlistoperation_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['promotion.ContentListOperation'], unique=True, primary_key=True)),
            ('target', self.gf('django.db.models.fields.related.ForeignKey')(related_name='+', to=orm['content.Content'])),
        ))
        db.send_create_signal(u'promotion', ['LockOperation'])

        # Adding model 'UnlockOperation'
        db.create_table(u'promotion_unlockoperation', (
            (u'contentlistoperation_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['promotion.ContentListOperation'], unique=True, primary_key=True)),
            ('target', self.gf('django.db.models.fields.related.ForeignKey')(related_name='+', to=orm['content.Content'])),
        ))
        db.send_create_signal(u'promotion', ['UnlockOperation'])

        # Adding model 'ContentList'
        db.create_table(u'promotion_contentlist', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=50)),
            ('length', self.gf('django.db.models.fields.IntegerField')(default=10)),
            ('data', self.gf('json_field.fields.JSONField')(default=[])),
        ))
        db.send_create_signal(u'promotion', ['ContentList'])

        # Adding model 'ContentListHistory'
        db.create_table(u'promotion_contentlisthistory', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('content_list', self.gf('django.db.models.fields.related.ForeignKey')(related_name='history', to=orm['promotion.ContentList'])),
            ('data', self.gf('json_field.fields.JSONField')(default=[])),
            ('date', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal(u'promotion', ['ContentListHistory'])


    def backwards(self, orm):
        # Deleting model 'ContentListOperation'
        db.delete_table(u'promotion_contentlistoperation')

        # Deleting model 'InsertOperation'
        db.delete_table(u'promotion_insertoperation')

        # Deleting model 'ReplaceOperation'
        db.delete_table(u'promotion_replaceoperation')

        # Deleting model 'LockOperation'
        db.delete_table(u'promotion_lockoperation')

        # Deleting model 'UnlockOperation'
        db.delete_table(u'promotion_unlockoperation')

        # Deleting model 'ContentList'
        db.delete_table(u'promotion_contentlist')

        # Deleting model 'ContentListHistory'
        db.delete_table(u'promotion_contentlisthistory')


    models = {
        user_model_label: {
            'Meta': {'object_name': User.__name__, 'db_table': "'%s'" % User._meta.db_table},
            User._meta.pk.attname: ('django.db.models.fields.AutoField', [], {'primary_key': 'True', 'db_column': "'%s'" % User._meta.pk.column}),
        },
        u'content.content': {
            'Meta': {'object_name': 'Content'},
            'authors': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['%s']" % user_orm_label, 'symmetrical': 'False'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '1024', 'blank': 'True'}),
            'feature_type': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('djbetty.fields.ImageField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'indexed': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'last_modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'auto_now': 'True', 'blank': 'True'}),
            'polymorphic_ctype': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'polymorphic_content.content_set'", 'null': 'True', 'to': u"orm['contenttypes.ContentType']"}),
            'published': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'default': "''", 'max_length': '50', 'blank': 'True'}),
            'subhead': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['content.Tag']", 'symmetrical': 'False', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '512'})
        },
        u'content.tag': {
            'Meta': {'object_name': 'Tag'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'polymorphic_ctype': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'polymorphic_content.tag_set'", 'null': 'True', 'to': u"orm['contenttypes.ContentType']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'promotion.contentlist': {
            'Meta': {'object_name': 'ContentList'},
            'data': ('json_field.fields.JSONField', [], {'default': '[]'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'length': ('django.db.models.fields.IntegerField', [], {'default': '10'}),
            'name': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50'})
        },
        u'promotion.contentlisthistory': {
            'Meta': {'object_name': 'ContentListHistory'},
            'content_list': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'history'", 'to': u"orm['promotion.ContentList']"}),
            'data': ('json_field.fields.JSONField', [], {'default': '[]'}),
            'date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        u'promotion.contentlistoperation': {
            'Meta': {'ordering': "['-when']", 'object_name': 'ContentListOperation'},
            'applied': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'content_list': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'operations'", 'to': u"orm['promotion.ContentList']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'polymorphic_ctype': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'polymorphic_promotion.contentlistoperation_set'", 'null': 'True', 'to': u"orm['contenttypes.ContentType']"}),
            'when': ('django.db.models.fields.DateTimeField', [], {})
        },
        u'promotion.insertoperation': {
            'Meta': {'ordering': "['-when']", 'object_name': 'InsertOperation', '_ormbases': [u'promotion.ContentListOperation']},
            'content': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'+'", 'to': u"orm['content.Content']"}),
            u'contentlistoperation_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': u"orm['promotion.ContentListOperation']", 'unique': 'True', 'primary_key': 'True'}),
            'index': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'lock': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        u'promotion.lockoperation': {
            'Meta': {'ordering': "['-when']", 'object_name': 'LockOperation', '_ormbases': [u'promotion.ContentListOperation']},
            u'contentlistoperation_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': u"orm['promotion.ContentListOperation']", 'unique': 'True', 'primary_key': 'True'}),
            'target': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'+'", 'to': u"orm['content.Content']"})
        },
        u'promotion.replaceoperation': {
            'Meta': {'ordering': "['-when']", 'object_name': 'ReplaceOperation', '_ormbases': [u'promotion.ContentListOperation']},
            'content': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'+'", 'to': u"orm['content.Content']"}),
            u'contentlistoperation_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': u"orm['promotion.ContentListOperation']", 'unique': 'True', 'primary_key': 'True'}),
            'lock': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'target': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'+'", 'to': u"orm['content.Content']"})
        },
        u'promotion.unlockoperation': {
            'Meta': {'ordering': "['-when']", 'object_name': 'UnlockOperation', '_ormbases': [u'promotion.ContentListOperation']},
            u'contentlistoperation_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': u"orm['promotion.ContentListOperation']", 'unique': 'True', 'primary_key': 'True'}),
            'target': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'+'", 'to': u"orm['content.Content']"})
        }
    }

    complete_apps = ['promotion']
